/*  _______              __
   / ___/ /  ___  __ _  / /  ___
  / /__/ _ \/ _ \/  ' \/ _ \/ _ \
  \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

// NodeBaseBottomSmoother.H
// adapted from BaseBottomSmoother by DFMartin, Sun, May 5, 2002
// petermc, 5 June 2002

#ifndef NODEBASEBOTTOMSMOOTHER_H
#define NODEBASEBOTTOMSMOOTHER_H

#include <iostream>
#include <math.h>
#include <assert.h>
#include <stdlib.h>
#include "REAL.H"
#include "IntVect.H"
#include "Box.H"
#include "DisjointBoxLayout.H"
#include "LevelData.H"
#include "NodeFArrayBox.H"
#include "ProblemDomain.H"
#include "NodeLevelOp.H"

/// Pure base class to encapsulate bottom smoother for NodeLevelOp-derived objects. 
/** The NodeBaseBottomSmoother class is a virtual base class for implementing 
    different bottomSmoother algorithms for NodeLevelOp-derived classes. It 
    takes a NodeLevelOp pointer as an argument during its definition, in order 
    to access the problem-dependent (equation-dependent) functionality of 
    the specific NodeLevelOp derivative class.  
*/ 

class NodeBaseBottomSmoother
{

public:

  /**
     \name Constructor and destructor
  */
  /*@{*/

  ///
  /** Default constructor.
   */
  NodeBaseBottomSmoother() {};

  ///
  /** Destructor.
   */
  virtual ~NodeBaseBottomSmoother() {};

  ///
  /** This gets around the "no virtual constructor" rule.
   */
  virtual NodeBaseBottomSmoother* new_bottomSmoother() const = 0;

  /*@}*/

  /**
     \name Parameter-setting functions
  */
  /*@{*/

  ///
  /** Control output to stdout.
   */
  void setVerbose( bool a_verbose ){ m_verbose = a_verbose ; } ;

  /*@}*/

  /**
     \name Data modification functions
  */
  /*@{*/

  ///
  /** Perform smoothing of L(<i>a_phi</i>) = <i>a_rhs</i>, where
      L is the operator defined in <i>a_levelop_ptr</i>,
      and the preconditioner is
      <i>a_levelop_ptr</i>->levelPreconditioner().
   */
  virtual void doBottomSmooth(/// data at this level, to be smoothed
                              LevelData<NodeFArrayBox>& a_phi,
                              /// right-hand side at this level
                              const LevelData<NodeFArrayBox>& a_rhs,
                              /// pointer to the operator
                              NodeLevelOp* a_levelop_ptr
                              ) = 0;

  /*@}*/

  /** enable output to stdout if <tt>true</tt>
   */
  bool m_verbose;
};

#endif
