/*  _______              __
   / ___/ /  ___  __ _  / /  ___
  / /__/ _ \/ _ \/  ' \/ _ \/ _ \
  \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

// NodeNoOpSmoother.H
// adapted from NoOpSmoother by DFMartin, Sun, May 5, 2002
// petermc, 5 June 2002

#ifndef NODENOOPSMOOTHER_H
#define NODENOOPSMOOTHER_H

#include <iostream>
#include <math.h>
#include <assert.h>
#include <stdlib.h>
#include "REAL.H"
#include "IntVect.H"
#include "Box.H"
#include "DisjointBoxLayout.H"
#include "LevelData.H"
#include "NodeFArrayBox.H"
#include "ProblemDomain.H"
#include "NodeBaseBottomSmoother.H"

/// Implements NoOp as a bottom smoother for LevelOp-derived classes
/** The NodeNoOpSmoother class implements the NoOp algorithm for 
    NodeLevelOp-derived classes
*/ 
class NodeNoOpSmoother : public NodeBaseBottomSmoother
{

public:

  /**
     \name Constructor and destructor
  */
  /*@{*/

  ///
  /** Default constructor.
   */
  NodeNoOpSmoother();

  ///
  /** Destructor.
   */
  virtual ~NodeNoOpSmoother();

  ///
  /** this gets around the "no virtual constructor" rule
   */
  virtual NodeBaseBottomSmoother* new_bottomSmoother() const;

  /*@}*/

  /**
     \name Data modification functions
  */
  /*@{*/

  /** Do nothing.
   */
  virtual void doBottomSmooth(LevelData<NodeFArrayBox>& a_phi,
                              const LevelData<NodeFArrayBox>& a_rhs,
                              NodeLevelOp* a_levelop_ptr);
  /*@}*/
};

#endif
