/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _localFuncs_
#define _localFuncs_
#include "LevelData.H"
#include "FArrayBox.H"
#include "ParmParse.H"
#include "Vector.H"
#include "SPMD.H"
#include "ProblemDomain.H"
#include "GhostBC.H"

/*
  set domain boundary conditions from input file
 */
int setDomainBC(DomainGhostBC& domghostbc, bool verbose, int ncomp = 1);

/*
  Set grid hierarchy, either from input file, or based on RHS
 */
int setGrids(Vector<DisjointBoxLayout>& vectGrids,
             Vector<ProblemDomain>&     vectDomain, 
             Vector<Real>&              vectDx, 
             Vector<int>&               vectRefRatio, 
             int& numlevels, bool verbose);

/*
  Set RHS on hierarchy from input file
 */
int setRHS(Vector<LevelData<FArrayBox>* >& vectRhs,
           Vector<Real>&                   vectDx, 
           Vector<ProblemDomain>&          vectDomain, 
           int numlevels,
           bool verbose);


void
tagCells(Vector<LevelData<FArrayBox>*>& vectRHS,
	 Vector<IntVectSet>& tagVect,
	 Vector<Real>& vectDx,
	 Vector<ProblemDomain>& vectDomain,
	 const Real refine_thresh,
	 const int tags_grow,
	 const int baselevel, 
	 int numlevels);

/*
   Output phi and rhs to file
 */
int outputData(const Vector<LevelData<FArrayBox>* >& vectPhi,
               const Vector<LevelData<FArrayBox>* >& vectRhs,
               const Vector<DisjointBoxLayout>& vectGrids,
               const Vector<ProblemDomain>& domain,
               const Vector<int>& vectRatio,
               Real dxCoarsest,
               int numlevels,
               bool verbose);

#endif

