/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "AMRLevel.H"
#include "AMRLevelWaveEquation.H"
#include "AMRLevelWaveEqnFactory.H"

AMRLevelWaveEqnFactory::AMRLevelWaveEqnFactory()
{
  setDefaultValues();
}

// Virtual constructor
AMRLevel* AMRLevelWaveEqnFactory::new_amrlevel() const
{
  // Make sure everything is defined
  assert(isDefined());

  // Create a new AMRLevelWaveEquation
  AMRLevelWaveEquation* amrWavePtr = new AMRLevelWaveEquation();

  // Set up new object
  amrWavePtr->CFL(m_cfl);
  amrWavePtr->domainLength(m_domainLength);
  amrWavePtr->refinementThreshold(m_refineThresh);
  amrWavePtr->tagBufferSize(m_tagBufferSize);
  amrWavePtr->initialDtMultiplier(m_initialDtMultiplier);
  amrWavePtr->verbosity(m_verbosity);
  amrWavePtr->IBC(m_wave_ibc);
  // Return it
  return (static_cast <AMRLevel*> (amrWavePtr));
}

AMRLevelWaveEqnFactory::~AMRLevelWaveEqnFactory()
{
}

// CFL number
void AMRLevelWaveEqnFactory::CFL(Real a_cfl)
{
  m_cfl = a_cfl;
  m_cflSet = true;
}

void AMRLevelWaveEqnFactory::verbosity(const int& a_verbosity)
{
  m_verbosity = a_verbosity;
}

// Physical dimension of the longest side of the domain
void AMRLevelWaveEqnFactory::domainLength(Real a_domainLength)
{
  m_domainLength = a_domainLength;
  m_domainLengthSet = true;
}

// Refinement threshold
void AMRLevelWaveEqnFactory::refinementThreshold(Real a_refineThresh)
{
  m_refineThresh = a_refineThresh;
  m_refineThreshSet = true;
}

// Tag buffer size
void AMRLevelWaveEqnFactory::tagBufferSize(int a_tagBufferSize)
{
  m_tagBufferSize = a_tagBufferSize;
  m_tagBufferSizeSet = true;
}

// Initial dt multiplier
void AMRLevelWaveEqnFactory::initialDtMultiplier(Real a_initialDtMultiplier)
{
  m_initialDtMultiplier = a_initialDtMultiplier;
  m_initialDtMultiplierSet = true;
}

// Check that everything is defined
bool AMRLevelWaveEqnFactory::isDefined() const
{
  return (m_cflSet &&
          m_domainLengthSet &&
          m_refineThreshSet &&
          m_tagBufferSizeSet &&
          m_initialDtMultiplierSet); 
}

// Some default values
void AMRLevelWaveEqnFactory::setDefaultValues()
{
  CFL(0.8);
  domainLength(1.0);
  refinementThreshold(0.2);
  tagBufferSize(2);
  initialDtMultiplier(0.1);
  m_verbosity = 0;
}
