/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cstdio>
#include <string>
using std::string;

#include "EBPatch.H"
#include "LoHiCenter.H"
#include "LoHiSide.H"

#include "EBOpsF_F.H"

void EBPatch::updateState(FArrayBox&       a_U,
                          Real&            a_maxWaveSpeed,
                          const Box&       a_box)
{
    assert (Spacedim == 2);
    FArrayBox dFTilde[SpaceDim];

    FArrayBox W;
    W.resize(grownBox,m_numComps);
    FArrayBox WPlus[SpaceDim];
    FArrayBox WMinus[SpaceDim];
    FArrayBox dW[SpaceDim];
    FORT_CONSTOPRIM(CHF_FRA[W],CHF_CONST_FRA[a_U],CHF_CONST_BOX[grownBox]);

    for (int dir = 0;dir < SpaceDim;dir++)
    {
      FArrayBox dW.resize(slopeBox,m_numComps);
      WPlus[dir].resize(slopeBox,m_numComps);
      WMinus[dir].resize(slopeBox,m_numComps);
      dFTilde[dir].resize(transDBox,m_numComps);
      FArrayBox FTilde(fluxTildeBox,m_numComps);
      FORT_EBSLOPE(CHF_CONST_FRA[W],
                   CHF_CONST_FIA1[mask],
                   CHF_FRA[dW],
                   CHF_CONST_BOX[slopeBox],
                   CHF_CONST_REAL[dt],
                   CHF_CONST_REAL[dx],
                   CHF_CONST_INT[m_numComps]
                   CHF_CONST_INT[dir]);
      FORT_EBNORMALPRED(
                   CHF_CONST_FRA[W],
                   CHF_CONST_FIA1[mask],
                   CHF_CONST_FRA[dW],
                   CHF_CONST_FRA[WPlus[dir]],
                   CHF_CONST_FRA[WMinus[dir]],
                   CHF_CONST_BOX[slopeBox],
                   CHF_CONST_REAL[dt],
                   CHF_CONST_REAL[dx],
                   CHF_CONST_INT[m_numComps]
                   CHF_CONST_INT[dir]);
      FORT_EBFLUX(
                   CHF_CONST_FRA[WPlus[dir]],
                   CHF_CONST_FRA[WMinus[dir]],
                   CHF_CONST_FIA[mask],
                   CHF_FRA[FTilde],
                   CHF_CONST_BOX[fluxTildeBox],
                   CHF_CONST_INT[m_numComps],
                   CHF_CONST_INT[dir]);
      Real dth = dt/2;
      FORT_EBDFLUX
                   CHF_CONST_FRA[FTilde],
                   CHF_FRA[dFTilde[dir]],
                   CHF_CONST_FIA1[mask],
                   CHF_CONST_BOX[transDFBox],
                   CHF_CONST_REAL[dth],
                   CHF_CONST_REAL[dx],
                   CHF_CONST_INT[m_numComps],
                   CHF_CONST_INT[dir]);
    }
    WPlus[0] -= dFTilde[1];
    WMinus[0] -= dFTilde[1];
    WPlus[1] -= dFTilde[0];
    WMinus[1] -= dFTilde[0];
    FArrayBox divF_NC(a_box,m_numComps);
    FArrayBox divF_C(a_box,m_numComps);
    FArrayBox Flux[SpaceDim];

    for (int dir = 0;dir < SpaceDim;dir++)
    {
      fluxBox = surroundingNodes(a_box,dir);
      FArrayBox Flux[dir](fluxBox,m_numComps);
      FArrayBox dFlux(a_box,m_numComps);
      FORT_EBFLUX(
                   CHF_CONST_FRA[WPlus[dir]],
                   CHF_CONST_FRA[WMinus[dir]],
                   CHF_CONST_FIA[mask],
                   CHF_FRA[Flux[dir]],
                   CHF_CONST_BOX[fluxBox],
                   CHF_CONST_INT[m_numComps],
                   CHF_CONST_INT[dir]);
      FORT_EBDFLUX(
                   CHF_CONST_FRA[Flux[dir]],
                   CHF_FRA[dFlux],
                   CHF_CONST_FIA1[mask],
                   CHF_CONST_BOX[a_box],
                   CHF_CONST_REAL[dt],
                   CHF_CONST_REAL[dx],
                   CHF_CONST_INT[m_numComps],
                   CHF_CONST_INT[dir]);
      divF_NC -= dFlux; 
      FORT_DIVFC(
               CHF_CONST_FRA[Flux[dir]],
               CHF_CONST_FRA[aper[dir]],
               CHF_CONST_FRA[centroid[dir]],
               CHF_CONST_FIA1[mask],
               CHF_FRA[divF],
               CHF_CONST_BOX[a_box],
               CHF_CONST_REAL[dt],
               CHF_CONST_REAL[dx],
               CHF_CONST_INT[dir],
               CHF_CONST_INT[m_numComps]);
    }
    FORT_DIVFCB(
               CHF_CONST_FRA[W],
               CHF_CONST_FRA[dW[0]],
               CHF_CONST_FRA[dW[1]],
               CHF_CONST_FRA[aper[0]],
               CHF_CONST_FRA[aper[1]],
               CHF_CONST_FRA[centbdry],
               CHF_CONST_FIA1[mask],
               CHF_CONST_REAL[dt],
               CHF_CONST_REAL[dx],
               CHF_CONST_INT[m_numComps]);
    FORT_UPDATE(
               CHF_CONST_FRA[divF],
               CHF_CONST_FRA[divF_NC],
               CHF_FRA[U],
               CHF_FRA1[kappa],
               CHF_CONST_REAL[dt],
               CHF_CONST_FIA1[mask],
               CHF_CONST_REAL[dt],
               CHF_CONST_REAL[dx],
               CHF_CONST_INT[m_numComps]);
}
