/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DMartin, Weds, July 21, 1999

#ifndef _BASEHELMHOLTZOP_H_
#define _BASEHELMHOLTZOP_H_

#include "LevelOp.H"
#include "GhostBC.H"

/// Pure-virtual base Helmholtz operator class (alpha*I + div*beta*grad)
/** Virtual base class to solve Helmholtz-type problems.  The raison
    d'Etre for this class is to allow for scaling of Alpha and Beta
    for any Helmholtz-type operator
 */
class BaseHelmholtzOp: public LevelOp
{
public:
  /// Scale the helmholtz coeficient alpha in (alpha*I + beta*laplacian)phi
  /**
     (alpha := a_scale*alpha)
  */
  virtual void scaleAlpha(Real a_scale) = 0;

  /// Scale the helmholtz coefficient beta in (alpha*I + beta*laplacian)phi
  /**
     (beta := a_scale*beta)
  */
  virtual void scaleBeta(Real a_scale) =0;

  /// set (or reset) the physical boundary conditions
  /**
   */
  virtual void setDomainGhostBC(const DomainGhostBC& a_dombc) = 0;

  /// Null Constructor
  BaseHelmholtzOp() {};

  /// Destructor
  virtual ~BaseHelmholtzOp() {};
};

#endif
