/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DFMartin, Sun, May 5, 2002

#ifndef _NOOPSMOOTHER_H_
#define _NOOPSMOOTHER_H_

#include <iostream>
#include <cmath>
#include <cassert>
#include <cstdlib>

#include "REAL.H"
#include "IntVect.H"
#include "Box.H"
#include "DisjointBoxLayout.H"
#include "LevelData.H"
#include "FArrayBox.H"
#include "ProblemDomain.H"

#include "BaseBottomSmoother.H"

/// Does nothing as a bottom smoother.
/**
    The NoOpSmoother class is a bottom smoother for LevelOp-derived
    classes which does nothing (mostly as a debugging test option)
*/
class NoOpSmoother: public BaseBottomSmoother
{
public:
  /// Null constructor
  NoOpSmoother();

  /// Destructor
  virtual ~NoOpSmoother();

  /// This gets around the "no virtual constructor" rule
  /**
   */
  virtual BaseBottomSmoother* new_bottomSmoother() const;

  /// Do (no) smoothing
  /**
      True to its name, this class does nothing in this functino
  */
  virtual void doBottomSmooth(LevelData<FArrayBox>&       a_phi,
                              const LevelData<FArrayBox>& a_rhs,
                              LevelOp*                    a_levelopPtr);

  // this doesn't really do anything, but is required by BaseBottomSmoother
  virtual void setConvergenceMetric(Real a_metric, int a_comp=0) {};

  /// this doesn't really do anything, but is required by BaseBottomSmoother
  virtual void setSolverTol(Real a_tol) {};

};

#endif
