/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "DerivStencil.H"

DerivStencil::DerivStencil()
{
    clear();
}
void
DerivStencil::define()
{
    clear();
}
DerivStencil::DerivStencil(const DerivStencil& a_dsin)
{
    clear();
    *this = a_dsin;
}

///make derivstencil empty
void
DerivStencil::clear()
{
    m_vectIV.resize(0);
    m_vectWgt.resize(0);
    isdefined = true;
}
DerivStencil::~DerivStencil()
{
    clear();
}

///return length of vectors
int 
DerivStencil::size() const
{
    assert(isdefined);
    assert(m_vectWgt.size() == m_vectIV.size());
    return(m_vectIV.size());
}

///get iv at ivec
const IntVect& 
DerivStencil::getIndex(int a_ivec) const
{
    assert(isdefined);
    assert(a_ivec >= 0);
    assert(a_ivec < m_vectIV.size());
    assert(m_vectWgt.size() == m_vectIV.size());
    return(m_vectIV[a_ivec]);
}

///get weight at ivec
const Real& 
DerivStencil::getWeight(int a_ivec) const
{
    assert(isdefined);
    assert(a_ivec >= 0);
    assert(a_ivec < m_vectIV.size());
    assert(m_vectIV.size()==m_vectWgt.size());
    return(m_vectWgt[a_ivec]);
}
    
/**
    add another set if the BAI is not in the 
    stencil already.  \\
    **Add the weight to the existing weight otherwise**
    */
void 
DerivStencil::accumulate(
    const IntVect& a_iv,  
    Real a_weight
    )    
{
    assert(isdefined);
    assert(m_vectIV.size()==m_vectWgt.size());
    bool isalreadythere = false;
    int ithere= -1;
    for(int ivect = 0; ivect < m_vectIV.size(); ivect++)
    {
        if(a_iv == m_vectIV[ivect])
        {
            isalreadythere = true;
            ithere = ivect;
            break;
        }
    }
    if(!isalreadythere)
    {
        m_vectWgt.push_back(a_weight);
        m_vectIV.push_back(a_iv);
    }
    else
    {
        m_vectWgt[ithere] += a_weight;
    }
}
const DerivStencil&
DerivStencil::operator=(const DerivStencil& a_dsin)
{
    assert(isdefined);
    if(&a_dsin != this)
    {
        m_vectIV = a_dsin.m_vectIV;
        m_vectWgt = a_dsin.m_vectWgt;
    }
    return *this;
}

const DerivStencil&
DerivStencil::operator+=(Real a_facin)
{
    assert(isdefined);
    assert(m_vectIV.size()==m_vectWgt.size());
    for(int ivec = 0; ivec < m_vectWgt.size(); ivec++)
    {
        m_vectWgt[ivec] += a_facin;
    }
    return *this;
}
                         
const DerivStencil&      
DerivStencil::operator-=(Real a_facin)
{
    assert(isdefined);
    assert(m_vectIV.size()==m_vectWgt.size());
    for(int ivec = 0; ivec < m_vectWgt.size(); ivec++)
    {
        m_vectWgt[ivec] -= a_facin;
    }
    return *this;
}
                         
const DerivStencil&      
DerivStencil::operator*=(Real a_facin)
{
    assert(isdefined);
    assert(m_vectIV.size()==m_vectWgt.size());
    for(int ivec = 0; ivec < m_vectWgt.size(); ivec++)
    {
        m_vectWgt[ivec] *= a_facin;
    }
    return *this;
}

const DerivStencil&      
DerivStencil::operator/=(Real a_facin)
{
    assert(isdefined);
    assert(m_vectIV.size()==m_vectWgt.size());
    for(int ivec = 0; ivec < m_vectWgt.size(); ivec++)
    {
        m_vectWgt[ivec] /= a_facin;
    }
    return *this;
}

