
/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL


#ifndef HDF5PORTABLE_H
#define HDF5PORTABLE_H

#include "Vector.H"
extern "C"
{
#ifdef HDF5
#include <hdf5.h>
#else
  typedef long hid_t ;
#endif
}
#include "Box.H"
#include "Interval.H"

template <class T>
void dataTypes(Vector<hid_t>& a_types, const T& a_dummySpecializationArg);

template <class T>
void dataSize(const T& item, Vector<int>& a_sizes, 
              const Box& box, const Interval& comps);

template <class T>
void write(const T& item, Vector<void*>& a_allocatedBuffers, 
           const Box& box, const Interval& comps);

template <class T>
void read(T& item, Vector<void*>& a_allocatedBuffers, const Box& box, 
          const Interval& comps);

template <class T>
const char* name(const T& a_dummySpecializationArg);

// These are the default template declarations for the HDF5 I/O
// template functions needed by T objects for binary IO.  The default
// implementations use the linearOut/linearIn/size functions of the
// T template classes to provide non-portable binary IO.
//     A user can add their own template specializations for
// portable binary IO abilities. 
//
//  To use this header, a user includes this file first, then
//  decalres their template specializations, then includes the CH_HDF5.H
//  header file.
// 
//
//
//  http://hdf.ncsa.uiuc.edu/HDF5/
//
// available datatypes in HDF5 
//
//            Example
//                                                  Corresponding C Type
//            H5T_NATIVE_CHAR
//                                                  char
//            H5T_NATIVE_UCHAR
//                                                  unsigned char
//            H5T_NATIVE_SHORT
//                                                  short
//            H5T_NATIVE_USHORT
//                                                  unsigned short
//            H5T_NATIVE_INT
//                                                  int
//            H5T_NATIVE_UINT
//                                                  unsigned
//            H5T_NATIVE_LONG
//                                                  long
//            H5T_NATIVE_ULONG
//                                                  unsigned long
//            H5T_NATIVE_FLOAT
//                                                  float
//            H5T_NATIVE_DOUBLE
//                                                  double
//
// and some special ones defined for Chombo:
//
//            H5T_NATIVE_REAL                       Real
//
//            box_id                                Box
//
//            intvext_id                            IntVect
#endif







